#!/bin/bash

#---------------------------------------------------------------------------
#
# Function 	:       wraps scheduler script
#
# Project       :       epages 6
#
# Copyright     :       epages Software GmbH 2009
#
# Version       :       Thu Aug 27 14:45:45 2009
#
# --------------------------------------------------------------------------

BN=`basename $0`
DN=`dirname $0`
DN=`cd $DN ; pwd`

EPAGES_PROJECT=epages6

unset ${!LC*}
export LANG=C
export LANGVAR=$LANG

#############################################################################
# usage & environment
#############################################################################

unset DEBUG_LINE
DEBUG_LINE[0]="$LINENO $0 $@"

usage () {
  echo "Usage: $BN [OPTIONS] SCRIPT_NAME [PROJECT]"
  echo "Arguments:"
  echo "  -D : debug script on command line"
  echo "  -d <DURATION>:"
  echo "   max duration the cronjob may take without a next instance reports an error"
  echo "   DURATION is one of:"
  echo "   <MINUTES> (e.g. -d 56)"
  echo "   <HOURS>:<MINUTES> (e.g. -d 7:30)"
  echo "   <DAYS>-<HOURS>:<MINUTES> (e.g. -d 3-00:00)"
  echo "  -w WRAP_SCRIPT"
  echo "   path to perl script (default: wrapScheduler.PL)"
  echo "  -i INI_FILE"
  echo "   path to ini file (default: \$EPAGES_CONFIG/Scheduler.conf)"
  echo "  -g GROUP"
  echo "   name of the group the script belongs to"
  echo "  -f"
  echo "   force execution of the script regardless of Scheduler.conf IsActive setting"
  echo "  SCRIPT_NAME:"
  echo "   either file  SCRIPT_NAME.env in \$EPAGES_CONFIG/Scheduler.d/"
  echo "   OR section SCRIPT_NAME in \$EPAGES_CONFIG/Scheduler.conf"
  echo "  PROJECT:"
  echo "   NAME as defined by /etc/default/NAME (default: $EPAGES_PROJECT)"
  echo "   OR own:NAME (use own, not ePages scheduler; set environment)"
  echo "   OR own: (use own, not ePages scheduler; don't set environment)"
  exit 1
}

unset MAX_AGE WRAP_SCRIPT INI_FILE GROUP FORCE DEBUG_CMDLINE
while getopts Dd:w:i:g:f c ; do
  case $c in
    D) DEBUG_CMDLINE=1 ;;
    d) MAX_AGE=$OPTARG ;;
    w) WRAP_SCRIPT=$OPTARG ;;
    i) INI_FILE=$OPTARG ;;
    g) GROUP=$OPTARG ;;
    f) FORCE=1 ;;
    *) usage ;;
  esac
done
shift `expr $OPTIND - 1`

[ $# -eq 0 ] && usage
RUN_SCRIPT=$1

# for convenience, allow -f to appear at the end of the command line like:
# wrapScheduler.sh [OPTIONS] SCRIPT_NAME [PROJECT] -f
if [[ $2 = -f ]] ; then
  FORCE=1
else
  EPAGES_PROJECT=${2:-$EPAGES_PROJECT}
  [[ $3 = -f ]] && FORCE=1
fi

if [ -n "$EPAGES_PROJECT" ] ; then
  EPAGES_DEFAULT=/etc/default/$EPAGES_PROJECT
  if [ ! -x $EPAGES_DEFAULT ] ; then
    echo "ERROR: Can't execute '$EPAGES_DEFAULT', exiting."
    exit 1
  fi
  DEBUG_LINE[1]="$LINENO . $EPAGES_DEFAULT"
  . "$EPAGES_DEFAULT"
  export EPAGES_PROJECT
fi
export AM_PROJECT_NAME

WRAP_SCRIPT=${WRAP_SCRIPT:-"$DN/wrapScheduler.PL"}
unset RUN_PATH

if [ "$EPAGES_PROJECT" = "${EPAGES_PROJECT#own:}" ] ; then
  AS_SCHEDULER_D=$EPAGES_CONFIG/Scheduler.d
  WS_SCHEDULER_D=$HTTPD_ROOT/conf/Scheduler.d

  case $RUN_SCRIPT in
    am-*) LOG_DIR="$EPAGES_LOG/Scheduler" ; SCHEDULER_D=/.no./.such./.file./.or./.directory/... ;;
    appserver-*) LOG_DIR="$EPAGES_LOG/Scheduler" ; SCHEDULER_D=/.no./.such./.file./.or./.directory/... ;;
    runserver-*) LOG_DIR="$EPAGES_LOG/Scheduler" ; SCHEDULER_D=/.no./.such./.file./.or./.directory/... ;;
    webserver-*) LOG_DIR="$HTTPD_ROOT/logs/Scheduler" ; SCHEDULER_D=$WS_SCHEDULER_D ;;
    root-*)      LOG_DIR="$HTTPD_ROOT/logs/Scheduler" ; SCHEDULER_D=/.no./.such./.file./.or./.directory/... ;;
    *) LOG_DIR="$EPAGES_LOG/Scheduler" ; unset SCHEDULER_D ;;
  esac

  [ -f "$EPAGES_CONFIG/Scheduler.d/$RUN_SCRIPT.env" ] && RUN_PATH="$EPAGES_CONFIG/Scheduler.d/$RUN_SCRIPT.env"
  [ -f "$SCHEDULER_D/$RUN_SCRIPT.env" ] && RUN_PATH="$SCHEDULER_D/$RUN_SCRIPT.env"

  if [ -n "$RUN_PATH" -a -z "$SCHEDULER_D" ] ; then
    echo $BN: SCRIPT_NAME does not start with root or {app,web,run}server- ; usage
  fi
fi

#############################################################################
# debugging
#############################################################################

unset COMMAND OPTIONS LOOP RECIPIENT DEBUG DEBUG_CRON ISACTIVE

FILE="$EPAGES_CONFIG"/Scheduler.conf

if [ -n "$RUN_PATH" ] ; then
  # shell script

  [ ! -r "$RUN_PATH" ] && { echo "ERROR: Can't read '$RUN_PATH', exiting.";  exit 1; }

  . "$RUN_PATH"

  # AD-3567: reimplement logic from wrapScheduler.PL for sh scripts here as well
  ISACTIVE_JOB=$ISACTIVE
  ISACTIVE_GLOBAL=$(/var/epages/ini-config.sh -f "$FILE" -s LOOPS -k IsActive -g 2>/dev/null)
  ISACTIVE_LOCATION=$(/var/epages/ini-config.sh -f "$FILE" -s LOOPS -k IsActiveLocation -g 2>/dev/null)
  [[ ${ISACTIVE_LOCATION:-1} = 1 || ${ISRUNNINGONPASSIVE:-0} = 1 ]] && ISACTIVE=1 || ISACTIVE=0
  [[ $ISACTIVE_GLOBAL = 1 && $ISACTIVE_JOB = 1 ]] || ISACTIVE=0

  if [ "$DEBUG" = "1" ] ; then
    DEBUG_LINE[2]="$LINENO . $RUN_PATH"
    DEBUG_CRON=1
  fi

else
  # perl script

  PATH=/var/epages:$PATH:/var/tmp

  [ ! -r "$FILE" ] && { echo "ERROR: Can't read '$FILE', exiting.";  exit 1; }

  DEBUG_LINE[2]="$LINENO ini-config.sh -f $FILE -s $RUN_SCRIPT -k Debug -g"
  DEBUG_CRON=$("ini-config.sh" -f "$FILE" -s $RUN_SCRIPT -k Debug -g 2>/dev/null)
  [ "$DEBUG_CRON" = "1" ] || unset DEBUG_CRON
  ISACTIVE=$("ini-config.sh" -f "$FILE" -s $RUN_SCRIPT -k IsActive -g 2>/dev/null)
fi

[[ -z $FORCE ]] || ISACTIVE=1

if [ -n "$DEBUG_CRON$DEBUG_CMDLINE" ] ; then
  DEBUG_CRON=/tmp/$RUN_SCRIPT.$(date +%F.%H:%M).debug
  rm -f $DEBUG_CRON
fi

debug () {
  local LINE_NUMBER=$(printf "%3d" $1)
  shift
  [ -n "$DEBUG_CRON" ] || return
  echo -e "wrapScheduler.sh:$LINE_NUMBER  // $@" >> $DEBUG_CRON
}

unset CROND_RUNNING
if [ -n "$DEBUG_CRON" ] ; then
  for i in 0 1 2 ; do
    [ -n "${DEBUG_LINE[$i]}" ] && debug ${DEBUG_LINE[$i]}
  done
  echo >> $DEBUG_CRON
  debug $LINENO "$DN/epagesScheduler.sh show | grep $RUN_SCRIPT"
  SHOW_CRON=$($DN/epagesScheduler.sh show 2>/dev/null | grep "\b$RUN_SCRIPT\b")
  [ -n "$SHOW_CRON" ] && echo "$SHOW_CRON" >> $DEBUG_CRON\
  || echo ">>> ERROR <<<: no such cronjob in epagesScheduler.sh: $RUN_SCRIPT" >> $DEBUG_CRON

  debug $LINENO "user $LOGNAME: crontab -l | grep $RUN_SCRIPT"
  SHOW_CRON=$(crontab -l | grep "\b$RUN_SCRIPT\b")
  [ -n "$SHOW_CRON" ] && echo "$SHOW_CRON" >> $DEBUG_CRON\
  || echo ">>> ERROR <<<: no such cronjob in crontab: $RUN_SCRIPT" >> $DEBUG_CRON

  debug $LINENO "user $LOGNAME: ps -p \$(pgrep crond) wwwue"
  CROND_RUNNING=$(pgrep crond | tr '\n' ',')
  [ -z "$CROND_RUNNING" ] && CROND_RUNNING=$(pgrep cron | tr '\n' ',')
  [ -n "$CROND_RUNNING" ] && ps -p ${CROND_RUNNING%,} wwwue >> $DEBUG_CRON\
  || echo ">>> ERROR <<<: cron daemon not running" >> $DEBUG_CRON
  echo >> $DEBUG_CRON
fi

HOST=${HOSTNAME%%.*}
TMP_FILE=/tmp/$HOST.$RUN_SCRIPT.~$$~

stop() {
  local RET=$1

  [[ -z $DEBUG_CMDLINE ]] || [[ -z $DEBUG_CRON ]] || cat $DEBUG_CRON
  rm -f "$TMP_FILE."*
  exit $RET
}

#############################################################################
# PERL SCRIPT
#############################################################################

export PERL

if [ -z "$RUN_PATH" ] ; then
  debug $LINENO "perl    : $(ls -d $PERL 2>&1)"
  debug $LINENO "wrapper : $(ls -d $WRAP_SCRIPT 2>&1)"
  [ -n "$INI_FILE" ] && debug $LINENO "ini file: $(ls -d $INI_FILE 2>&1)"
  debug $LINENO "log dir : $(ls -d $LOG_DIR 2>&1)"

  PATH=/var/epages:$PATH:/var/tmp:$DN
  INI_FILE=${INI_FILE:+" -ini '$INI_FILE'"}
  LOG_DIR=${LOG_DIR:+" -var '$LOG_DIR'"}
  MAX_AGE=${MAX_AGE:+" -maxage '$MAX_AGE'"}
  CMD="DEBUG_CRON=$DEBUG_CRON $PERL \"$WRAP_SCRIPT\"$INI_FILE$LOG_DIR$MAX_AGE $RUN_SCRIPT"
  [[ -z $DEBUG_CMDLINE ]] || CMD="$CMD -debug"
  [[ -z $FORCE ]] || CMD="$CMD -force"
  debug $LINENO "user $LOGNAME: $CMD"
  if [ -n "$DEBUG_CRON" ] ; then
    echo >> $DEBUG_CRON
    eval $CMD >> $DEBUG_CRON 2>&1
    RET=$?
  else
    eval $CMD
    RET=$?
  fi
  
  [ "$ISACTIVE" = 1 ] || stop $RET

  [ -f /var/log/cron ] && CRON_LOG=/var/log/cron || CRON_LOG=/var/log/messages
  debug $LINENO "grep $RUN_SCRIPT $CRON_LOG | tail -1"
  if [ -r $CRON_LOG -a -n "$DEBUG_CRON" -a -n "$CROND_RUNNING" ] ; then
    grep "\b$RUN_SCRIPT\b" $CRON_LOG | tail -1 >> $DEBUG_CRON
  elif [ -n "$DEBUG_CRON" -a -n "$CROND_RUNNING" ] ; then
    debug $LINENO "user $LOGNAME: $CRON_LOG (readable: 0)"
  fi

  stop $RET
fi

#############################################################################
# SHELL SCRIPT
#############################################################################

MYSELF=${SUDO_USER:-$USER}
MYSELF=${MYSELF:-$LOGNAME}

LOG_DIR=${LOG_DIR:-$HOME}
mkdir -p $LOG_DIR
LOG_FILE=$LOG_DIR/$HOST.$RUN_SCRIPT

RECIPIENT=${RECIPIENT:-$MYSELF@localhost}
[ "$RECIPIENT" = "@localhost" ] && RECIPIENT=root@localhost

#****************************************************************************
# write_mail_header
#****************************************************************************

write_mail_header() {
    echo -e "*** Output from program $RUN_SCRIPT ***\n" > "$TMP_FILE.head"
    echo "COMMAND: $COMMAND $OPTIONS > $TMP_FILE.run 2>&1" >> "$TMP_FILE.head"
    [ -n "$LOOP" ] && echo "LOOP: $LOOP" >> "$TMP_FILE.head"
    echo -e "DATE: `date`\nHOSTNAME: $HOST\nUSERNAME: $MYSELF\n" >> "$TMP_FILE.head"
}

#****************************************************************************
# send_mail
#****************************************************************************

send_mail() {
  local S="$HOST: Output from program $RUN_SCRIPT"
  [[ -z $AM_PROJECT_NAME ]] || S="$AM_PROJECT_NAME: $S"

  local SENDFILE=$1
  local LOGFILE=$2
  cat "$SENDFILE" > "$LOGFILE"

  local CONF=$EPAGES_CONFIG/Scheduler.conf
  # for shell scripts with $RUN_SCRIPT.env we use $RUN_SCRIPT.ini if it exists
  # instead of $EPAGES_CONFIG/Scheduler.conf
  if [[ -n $RUN_PATH && -f ${RUN_PATH%.env}.ini ]] ; then
    CONF=${RUN_PATH%.env}.ini
  fi

  FSIZE=`ls -l $SENDFILE | awk '{print $5}'`
  if [ -n "$FSIZE" ] && [ $FSIZE -gt 200000 ]  ; then
    bzip2 -9 $SENDFILE
    debug $LINENO "$PERL $DN/SendProgramOutput.pl -ini $CONF -section $RUN_SCRIPT -subject '$S' -file $SENDFILE.bz2 -attachment >> $LOGFILE"
    $PERL "$DN/SendProgramOutput.pl" -ini "$CONF" -section $RUN_SCRIPT -subject "$S" -file "$SENDFILE.bz2" -attachment >> "$LOGFILE" 2>&1
  else
    debug $LINENO "$PERL $DN/SendProgramOutput.pl -ini $CONF -section $RUN_SCRIPT -subject '$S' -file $SENDFILE >> $LOGFILE"
    $PERL "$DN/SendProgramOutput.pl" -ini "$CONF" -section $RUN_SCRIPT -subject "$S" -file "$SENDFILE" >> "$LOGFILE" 2>&1
  fi

  [ -n "$DEBUG_CRON" ] && echo >> $DEBUG_CRON

  rm -f "$SENDFILE" "$SENDFILE.bz2" 2>/dev/null
  echo >> "$LOGFILE"
}

#****************************************************************************
# exit_if_running_process_found
#****************************************************************************

#----------------------------------------------------------------------------
# get_days
#----------------------------------------------------------------------------

get_days () {
  echo $1 | awk -F- '{sub(/^0+/,""); if (/^[0-9]+-/) {print $1} else {print 0}}'
}

#----------------------------------------------------------------------------
# get_hours
#----------------------------------------------------------------------------

get_hours () {
  echo $1 | sed 's/.*-//' | awk -F\: '{sub(/^0+/,"",$1); if ($2 ~ /^[0-9]+$/) {print $1} else {print 0}}'
}

#----------------------------------------------------------------------------
# get_minutes
#----------------------------------------------------------------------------

get_minutes () {
  echo $1 | sed 's/.*://' | awk '{sub(/^0+/,""); if (/^[0-9]+$/) {print $0} else {print 0}}'
}

#----------------------------------------------------------------------------
# process_is_young
#----------------------------------------------------------------------------

process_is_young () {
  ELAPSED_D=$1 ; ELAPSED_H=$2 ; ELAPSED_M=$3
  MAX_D=$4 ; MAX_H=$5 ; MAX_M=$6

  [ $# -ne 6 -o `expr "$1$2$3$4$5$6" : '[0-9][0-9]*$'` -lt 6 ] && return 1

  MAX_H=`expr $MAX_H + \( $MAX_M / 60 \)`
  MAX_M=`expr $MAX_M % 60`
  MAX_D=`expr $MAX_D + \( $MAX_H / 24 \)`
  MAX_H=`expr $MAX_H % 24`

  [ $ELAPSED_D -gt $MAX_D ] && return 1
  [ $ELAPSED_D -lt $MAX_D ] && return 0
  [ $ELAPSED_H -gt $MAX_H ] && return 1
  [ $ELAPSED_H -lt $MAX_H ] && return 0
  [ $ELAPSED_M -gt $MAX_M ] && return 1
  [ $ELAPSED_M -lt $MAX_M ] && return 0
  return 1
}

#----------------------------------------------------------------------------
# exit_if_running_process_found
#----------------------------------------------------------------------------

PS_FORMAT=user,pid,ppid,nice,pri,pcpu,time,pmem,vsz,etime,args
COLUMNS=1024 ; export COLUMNS

exit_if_running_process_found () {
  PID=`cat $1 2>/dev/null`
  debug $LINENO "cat $1 -> PID=$PID"
  PID=`ps -p$PID -opid= 2>/dev/null`
  debug $LINENO "ps -p$PID -opid= 2>/dev/null -> PID=$PID"

  if [ -z "$PID" ] ; then
    rm -f "$1" 2>/dev/null
    return 0
  fi

  if [ -n "$MAX_AGE" ] ; then
    ELAPSED=`ps -p$PID -oetime= | sed 's/:[^:]*$//'`
    ELAPSED_D=`get_days $ELAPSED`
    ELAPSED_H=`get_hours $ELAPSED`
    ELAPSED_M=`get_minutes $ELAPSED`
    MAX_D=`get_days $MAX_AGE`
    MAX_H=`get_hours $MAX_AGE`
    MAX_M=`get_minutes $MAX_AGE`
    process_is_young $ELAPSED_D $ELAPSED_H $ELAPSED_M $MAX_D $MAX_H $MAX_M && Y=1 || Y=0
    debug $LINENO "process_is_young $ELAPSED_D $ELAPSED_H $ELAPSED_M $MAX_D $MAX_H $MAX_M -> $Y"
    [ $Y = 1 ] && stop 1
  fi

  echo Outdated scheduler ID file found: > "$TMP_FILE.mail"
  ls -l "$1" >> "$TMP_FILE.mail" 2>&1
  echo >> "$TMP_FILE.mail"
  echo Check the corresponding processes and stop them if necessary. >> "$TMP_FILE.mail"
  echo Delete the scheduler ID file, if the processes have stopped. >> "$TMP_FILE.mail"
  echo >> "$TMP_FILE.mail"
  ps -p$PID -o$PS_FORMAT >> "$TMP_FILE.mail"
  echo >> "$TMP_FILE.mail"
  cat "$TMP_FILE.head" "$TMP_FILE.run" >> "$TMP_FILE.mail" 2>/dev/null
  debug $LINENO "send_mail $TMP_FILE.mail $LOG_FILE.log"
  [ -n "$DEBUG_CRON" ] && echo >> $DEBUG_CRON
  send_mail "$TMP_FILE.mail" "$LOG_FILE.log"
  stop 1
}

exit_if_running_process_found "$LOG_FILE.pid"
write_mail_header

echo $$ > "$LOG_FILE.pid"
rm -f "$TMP_FILE.run"

#****************************************************************************
# run shell script
#****************************************************************************

[ "$ISACTIVE" = 1 ] || stop 0

expr "$COMMAND" : '.*[a-zA-Z0-9_]' >/dev/null 2>&1
if [ $? -ne 0 ] ; then
  debug $LINENO No valid variable COMMAND set in file: $RUN_PATH
  echo No valid variable COMMAND set in file: > "$TMP_FILE.run"
  ls -l "$RUN_PATH" >> "$TMP_FILE.run" 2>&1
elif [ -z "$LOOP" ] ; then
  debug $LINENO "nice -n 19 $COMMAND $OPTIONS"
  eval "nice -n 19 $COMMAND $OPTIONS" > "$TMP_FILE.run" 2>&1
else
  echo $LOOP | tr ',' '\n' | while read loop ; do
    debug $LINENO "nice -n 19 $COMMAND $OPTIONS $loop"
    eval "nice -n 19 $COMMAND $OPTIONS $loop" >> "$TMP_FILE.run" 2>&1
  done
fi
[ -n "$DEBUG_CRON" ] && echo >> $DEBUG_CRON

if [ -s "$TMP_FILE.run" ] ; then
  cat "$TMP_FILE.head" "$TMP_FILE.run" > "$TMP_FILE.mail"
  echo "ENDDATE: `date`" >> "$TMP_FILE.mail"
  debug $LINENO "send_mail $TMP_FILE.mail $LOG_FILE.log"
  send_mail $TMP_FILE.mail $LOG_FILE.log
fi
rm -f "$LOG_FILE.pid"

[ -f /var/log/cron ] && CRON_LOG=/var/log/cron || CRON_LOG=/var/log/messages
debug $LINENO "grep $RUN_SCRIPT $CRON_LOG | tail -1"
if [ -r $CRON_LOG -a -n "$DEBUG_CRON" -a -n "$CROND_RUNNING" ] ; then
  grep "\b$RUN_SCRIPT\b" $CRON_LOG | tail -1 >> $DEBUG_CRON
elif [ -n "$DEBUG_CRON" -a -n "$CROND_RUNNING" ] ; then
  debug $LINENO "user $LOGNAME: $CRON_LOG (readable: 0)"
fi

stop 0
#
#	end
#
