#==============================================================================
# package      DE_EPAGES::CrossSelling::API::Constants
# state        private
#------------------------------------------------------------------------------
# description  Some constants used for CSV export/import
#==============================================================================
package DE_EPAGES::CrossSelling::API::Constants;
use base Exporter;

use strict;
our @EXPORT = qw(
    $CSV_SPECIAL_CROSSSELLINGITEMS
    $CSV_SPECIAL_CROSSSELLINGITEMS_UNITY
    @CSV_WARNING_CODES_CROSSSELLING
    @CROSSSELLING_CLASSES
    $BUNDLEATTRIBUTE_MAP
    UNITY_CROSS_SELLING_NOTIFICATION_THRESHOLD
);
use vars qw ( $CSV_SPECIAL_CROSSSELLINGITEMS
              $CSV_SPECIAL_CROSSSELLINGITEMS_UNITY
              @CSV_WARNING_CODES_CROSSSELLING
              @CROSSSELLING_CLASSES
              $BUNDLEATTRIBUTE_MAP
            );

# attributes that are excluded from csv export/import
# 0 - excluded attributes
# 1 - included attributes (even though they are not marked as exportable)
$CSV_SPECIAL_CROSSSELLINGITEMS = {
    0 => [ qw( GUID Inherit CreationDate Alias FilePathPart ) ],
    1 => [ qw( Class ) ],
};

# attributes that are excluded from csv export/import for Unity shops
# 0 - excluded attributes
# 1 - included attributes (even though they are not marked as exportable)
$CSV_SPECIAL_CROSSSELLINGITEMS_UNITY = {
    0 => [ qw( GUID Inherit CreationDate Alias FilePathPart
               Comment ) ],
    1 => [ qw( Class ) ],
};

# possible warnings during CSV import
@CSV_WARNING_CODES_CROSSSELLING = qw(
                                    CROSSSELLING_EMPTY_PRODUCT
                                    CROSSSELLING_NO_PRODUCT
                                    CROSSSELLING_EMPTY_TARGETPRODUCT
                                    CROSSSELLING_NO_TARGETPRODUCT
                                    NO_MAND_VALUE
                                    WRONG_CLASS
                                    ASSIGN_PRODUCT_EQUAL
                                    PRODUCT_NOT_BUNDLEPRODUCT
                                    TARGETPRODUCT_IS_BUNDLEPRODUCT
                                    TARGETPRODUCT_IS_CUSTOMIZABLE
                                    NOT_SAME_TAXCLASS
                                    NO_FEATURE
                                    VOID_FIELD
                                    NO_CROSSSELLINGITEMS
                                  );

# allowed classes for export / import
@CROSSSELLING_CLASSES = qw(
                            CrossSelling
                            Accessory
                            ProductComparison
                            BundleProductMap
                        );
$BUNDLEATTRIBUTE_MAP = {
    'Product' => 'BundleProduct',
    'TargetProduct' => 'BundledProduct',
};

#========================================================================================
# global       UNITY_CROSS_SELLING_NOTIFICATION_THRESHOLD | minimal number
#                   of products to display the cross-selling notification | int
#========================================================================================
use constant UNITY_CROSS_SELLING_NOTIFICATION_THRESHOLD => 15;

1;

