#========================================================================================
# package      DE_EPAGES::CrossSelling::Hooks::CrossSelling
# state        private
# description  sets the cross-selling feature value.
#========================================================================================
package DE_EPAGES::CrossSelling::Hooks::CrossSelling;

use strict;

#========================================================================================
# function     OnInsertCrossSelling
# state        private
#----------------------------------------------------------------------------------------
# syntax       OnInsertCrossSelling( $hParams );
#----------------------------------------------------------------------------------------
# description  Disbales the notification for cross-selling once a cross-selling product
#               has been added.
#               (Hook OBJ_InsertCrossSelling)
#----------------------------------------------------------------------------------------
# input        $hParams | hash parameters, key is
#               <ul>
#                   <li>Object - CrossSelling object - object
#               </ul> | ref.hash.*
#========================================================================================
sub OnInsertCrossSelling {
    my ($hParams) = @_;
    my $CrossSelling = $hParams->{'Object'};
    my $Shop = $CrossSelling->getSite();

    # we do the following actions only for unity shops
    if ($Shop->get('IsUnityRegistered')) {

        my $CrossSellingNotification
            = $Shop->folder('ShopNotifications')->child('CrossSelling');

        return unless defined $CrossSellingNotification;
        return unless $CrossSellingNotification->get('IsActive');

        # deactivate notification if cross selling is set up
        # add shop object to avoid reactivation
        $CrossSellingNotification->addObject($Shop);
        $CrossSellingNotification->deactivate();
    }

    return;
}

1;
