#========================================================================================
# package      DE_EPAGES::CrossSelling::Hooks::Feature
# state        public
# description  hook handler for Feature objects
#========================================================================================
package DE_EPAGES::CrossSelling::Hooks::Feature;

use DE_EPAGES::CrossSelling::API::Cartridge;
use DE_EPAGES::CrossSelling::API::Constants qw ( UNITY_CROSS_SELLING_NOTIFICATION_THRESHOLD );
use DE_EPAGES::Object::API::Factory qw ( LoadClassByAlias LoadObject );
use strict;

#========================================================================================
# function     OnAfterUpdateShopFeature
#----------------------------------------------------------------------------------------
# syntax       OnAfterUpdateShopFeature( $hParams );
#----------------------------------------------------------------------------------------
# description  OnAfterUpdateShopFeature is triggered, initializes the notification for
#               cross-selling
#----------------------------------------------------------------------------------------
# input        $hShopFeature - feature object, used keys are
#               <ul>
#                   <li>FeatureID - feature id            | integer</li>
#                   <li>ShopID    - shop id               | integer</li>
#                   <li>MaxValue  - maximum feature value | int</li>
#               </ul> | ref.hash.*
#========================================================================================
sub OnAfterUpdateShopFeature {
    my ($hShopFeature) = @_;
    my $Feature = LoadObject($hShopFeature->{'FeatureID'});

    if ($Feature->alias eq 'ManualCrossSelling') {

        my $Shop = LoadObject($hShopFeature->{'ShopID'});
        # only for Unity shops
        if ($Shop->get('IsUnityRegistered')) {

            my $CrossSellingNotification
                = $Shop->folder('ShopNotifications')->child('CrossSelling');
            return unless defined $CrossSellingNotification;

            if ($hShopFeature->{'MaxValue'}) {
                # check if the shop has more than 15 products
                if ($Shop->featureCurrentValue('Products') >= UNITY_CROSS_SELLING_NOTIFICATION_THRESHOLD &&
                    $Shop->featureMaxValue('ManualCrossSelling')) {

                    my $CrossSellingClass = LoadClassByAlias('CrossSelling');
                    # activate notification if there are no cross selling products
                    # present in the shop
                    $CrossSellingNotification->addObject($Shop)
                        if(scalar @{$CrossSellingClass->findObjects('BySiteIDClassID',
                            [$Shop->id, $CrossSellingClass->id])} == 0);
                }
            } else {
                # deactivate notification if feature is unset
                $CrossSellingNotification->deleteObject($Shop);
                $CrossSellingNotification->deactivate();
            }
        }
    }

    return;
}


1;