#========================================================================================
# package      DE_EPAGES::CrossSelling::Hooks::Shop
# state        public
# description  hook handler for shop objects
#========================================================================================
package DE_EPAGES::CrossSelling::Hooks::Shop;

use DE_EPAGES::CrossSelling::API::Cartridge;
use DE_EPAGES::CrossSelling::API::Constants qw ( UNITY_CROSS_SELLING_NOTIFICATION_THRESHOLD );
use DE_EPAGES::Object::API::Factory qw ( LoadClassByAlias );
use DE_EPAGES::Object::API::XML::ObjectImportHandler qw ( ImportObjects );
use strict;

#========================================================================================
# function     OnSEOCockpit
#----------------------------------------------------------------------------------------
# syntax       OnSEOCockpit( $hParams );
#----------------------------------------------------------------------------------------
# description  Supply cross-selling data for SEO cockpit.
#----------------------------------------------------------------------------------------
# input        $hParams | objects
#               <ul>
#                   <li>Shop - shop object - object
#                   <li>Values - value hash to fill/change data - ref.hash
#               </ul> | ref.hash.*
#========================================================================================
sub OnSEOCockpit {
    my ($hParams) = @_;
    my $Shop = $hParams->{'Shop'};

    my $hValues = $hParams->{'Values'};

    if($Shop->featureMaxValue('AutoCrossSelling')) {
        my $ACSRating = ($Shop->get('AutoCrossSellingCollectData') && $Shop->get('AutoCrossSellingIsVisible')) ? 1 : 2;

        $hValues->{'AutoCrossSelling'} = {
            'Rating' => $ACSRating,
            'RatingShades' => [ qw ( green red ) ],
            'Percentage'   => 0.1,
        };
    }
}

#========================================================================================
# function     OnBeforeCreateUnityShop
#----------------------------------------------------------------------------------------
# syntax       OnBeforeCreateUnityShop( $hParams );
#----------------------------------------------------------------------------------------
# description  OnBeforeCreateUnityShop is triggered, initializes the notification for
#               cross selling
#----------------------------------------------------------------------------------------
# input        $hParams | hook arguments, used keys are
#               <ul>
#                   <li>Shop - shop object | object</li>
#               </ul> | ref.hash.*
#========================================================================================
sub OnBeforeCreateUnityShop {
    my ($hParams) = @_;
    my $Shop = $hParams->{'Object'};

    # import notifications
    my $Cartridge = DE_EPAGES::CrossSelling::API::Cartridge->new;
    ImportObjects($Cartridge->xmlDirectory().'/ShopNotifications.xml', $Shop->pathString);

    # check if the shop has more than 15 products
    if ($Shop->featureCurrentValue('Products') >= UNITY_CROSS_SELLING_NOTIFICATION_THRESHOLD &&
        $Shop->featureMaxValue('ManualCrossSelling')) {

        my $CrossSellingClass = LoadClassByAlias('CrossSelling');
        # check if there are no cross selling products present in the shop
        if(scalar @{$CrossSellingClass->findObjects('BySiteIDClassID',
            [$Shop->id, $CrossSellingClass->id])} == 0) {

            # activate notification
            my $CrossSellingNotification
                = $Shop->folder('ShopNotifications')->child('CrossSelling');
            $CrossSellingNotification->addObject($Shop);
        }
    }

    return;
}


1;