package DE_EPAGES::CrossSelling::t::ShopNotification;
use base DE_EPAGES::Shop::API::Test;

use Test::More;
use strict;
use DE_EPAGES::Object::API::Factory qw (
    LoadRootObject
    LoadObjectByPath
);
use DE_EPAGES::Object::API::XML::ObjectImportHandler qw ( ImportObjects );
use DE_EPAGES::CrossSelling::API::CrossSelling qw ( AddCrossSelling );

my $ROOTPATH = '/Test';
my $SHOPPATH = '/Test/TestShop/';
local $ENV{'EPAGES_TESTSITE'} = 'Store';

sub test {

    _testCrossSellingNotification();
    done_testing();
}

sub _testCrossSellingNotification {

    my $Shop = LoadObjectByPath($SHOPPATH);
    my $NotificationFolder = $Shop->folder('ShopNotifications');

    ok( $NotificationFolder->existsChild('CrossSelling'),
        'ShopNotification CrossSelling exists' );
    my $CrossSellingNotification = $NotificationFolder->child('CrossSelling');
    is( $CrossSellingNotification->get('IsActive'), 0,
        'ShopNotification CrossSelling is inactive with '
        . $Shop->featureCurrentValue('Products') . ' products');

    my $Product1 = LoadObjectByPath('/Test/TestShop/Products/MyRedNose');
    my $Product2 = LoadObjectByPath('/Test/TestShop/Products/MyYellowNose');
    my $Product3 = $Product1->clone({'Alias' => 'MyGreenNose'});
    is( $CrossSellingNotification->get('IsActive'), 1,
        'ShopNotification CrossSelling is active with '
        . $Shop->featureCurrentValue('Products') . ' products and no cross selling');

    my $Product4 = $Product1->clone({'Alias' => 'MyBlueNose'});
    is( $CrossSellingNotification->get('IsActive'), 1,
        'ShopNotification CrossSelling is still active with '
        . $Shop->featureCurrentValue('Products') . ' products and no cross selling');

    note 'Unset shop feature "ManualCrossSelling"';
    $Shop->featureMaxValue('ManualCrossSelling', 0);
    is( $CrossSellingNotification->get('IsActive'), 0,
        'ShopNotification CrossSelling is inactive without the feature "ManualCrossSelling"');

    note 'Re-set shop feature "ManualCrossSelling"';
    $Shop->featureMaxValue('ManualCrossSelling', 1);
    is( $CrossSellingNotification->get('IsActive'), 1,
        'ShopNotification CrossSelling is active again after activating the feature "ManualCrossSelling" with '
        . $Shop->featureCurrentValue('Products') . ' products and no cross selling in the shop');

    $Product3->delete;
    $Product4->delete;
    is( $CrossSellingNotification->get('IsActive'), 1,
        'ShopNotification CrossSelling is still active with '
        . $Shop->featureCurrentValue('Products') . ' products and no cross selling');


    AddCrossSelling($Product1, [$Product2], 'CrossSelling');
    is( $CrossSellingNotification->get('IsActive'), 0,
        'ShopNotification CrossSelling is inactive with '
        . $Shop->featureCurrentValue('Products') . ' products and added cross selling');
    is( $CrossSellingNotification->get('ShopNotificationObjectIDString'), $Shop->id,
        'The shop got added as (only) notification object');

    $Product3 = $Product1->clone({'Alias' => 'MyGreenNose'});
    is( $CrossSellingNotification->get('IsActive'), 0,
        'ShopNotification CrossSelling is still inactive with '
        . $Shop->featureCurrentValue('Products') . ' products and added cross selling');
}

# be sure having reseted IsUnityRegistered
# otherwise we can not delete the shop because of remote connection to the SF does not exists
sub cleanup {
    my $self = shift;

    return if $ENV{'SKIP_CLEANUP'};

    my $System = LoadRootObject();
    if( $System->existsChild($self->testFolder) ) {
        my $Folder = $System->child($self->testFolder);
        $Folder->child($self->shopAlias)->set({'IsUnityRegistered' => 0}) if $Folder->existsChild($self->shopAlias);
    }

    # call the real cleanup
    $self->SUPER::cleanup;
}

sub init {
    my $self = shift;

    $self->SUPER::init;
    die 'Shop alias is not TestShop' unless $self->shop->alias eq 'TestShop';

    note 'importing import.xml';
    ImportObjects('import.xml', $self->shop->parent->pathString);
}

__PACKAGE__->runScript;