#==============================================================================
# package      DE_EPAGES::Forum::UI::GuestBook
# base         DE_EPAGES::Forum::UI::Forum
# state        private
#==============================================================================
package DE_EPAGES::Forum::UI::GuestBook;
use base DE_EPAGES::Forum::UI::Forum;

use strict;
use DE_EPAGES::Core::API::EpagesConfig qw ( EpagesConfig );
use DE_EPAGES::Object::API::Factory qw( LoadClassByAlias );
use DE_EPAGES::Search::API::Object::PagerSearch;
use DE_EPAGES::Search::API::SearchDefinition  qw (GetSearchDefinition);

use constant LENGTH => 30;

sub AddThread {
    my $self = shift;
    my ($Servlet) = @_;
    my $DisableReCaptcha = 1;
    my $GuestBook = $Servlet->object;
    my $Form = $Servlet->form;
    my $ThreadsCount = $Servlet->object->get('ThreadsCount');
    my $MaxNumGuestbookReplies = EpagesConfig()->get('DE_EPAGES::Forum', 'MaxNumGuestbookReplies', '10000');

    #spam check - check if field senderName has value,
    #this field is only for spam protection it is hidden for the customer
    return if ($Form->value('SenderName') or ($ThreadsCount >= $MaxNumGuestbookReplies));

    # check message form field
    if(!$self->CheckFormInput($Form, 'Message')) {
        $Servlet->form->executeFormError;
        return;
    }

    # no subject in guestbook entries, so generate alias from message field
    $Form->{'Values'}->{'Subject'}->[0] = $GuestBook->uniqueChildAlias(substr($Form->{'Values'}->{'Message'}->[0], 0, LENGTH));
    $self->SUPER::AddThread($Servlet, $DisableReCaptcha);

    return;
}

sub View {
    my $self = shift;
    my ($Servlet) = @_;

    my $ThreadsCount = $Servlet->object->get('ThreadsCount');
    my $MaxNumGuestbookReplies = EpagesConfig()->get('DE_EPAGES::Forum', 'MaxNumGuestbookReplies', '10000');

    my $TooManyReplies = ($ThreadsCount >= $MaxNumGuestbookReplies) ? 1 : 0;

    $Servlet->vars('TooManyReplies', $TooManyReplies);

    return $self->_searchPosts( $Servlet, $Servlet->isEditorMode ? 0 : 1 );
}

sub MBOViewChildObjects {
    my $self = shift;
    my ($Servlet) = @_;

    return $self->_searchPosts( $Servlet, 0 );
}

sub _searchPosts {
    my $self = shift;
    my ($Servlet, $OnlyVisiblePosts) = @_;

    my $GuestBook = $Servlet->object;

    my $SearchDefinition = GetSearchDefinition(LoadClassByAlias('ForumPost'), 'GuestBookEntries');
    my %Options = (
        'Servlet' => $Servlet,
        'SearchDefinition' => $SearchDefinition,
        'OrderBy' => 'CreationDate',
        'OrderDesc' => 1,
        'Parameters' => {
            'ForumID'   => $GuestBook->id,
        },
    );
    $Options{'Parameters'}->{'IsVisible'} = 1 if $OnlyVisiblePosts;

    return DE_EPAGES::Search::API::Object::PagerSearch->new(\%Options)->page;
}

1;
