#========================================================================================
# package      DE_EPAGES::Newsletter::DAL::Newsletter
# state        private
# description  database interface for tables:
#               <ul>
#                   <li>newsletteradressee
#               </ul>
#========================================================================================
package DE_EPAGES::Newsletter::DAL::Newsletter;
use base qw ( Exporter );

use strict;

use DE_EPAGES::Core::API::PerlTools qw ( UsePackage );
use DE_EPAGES::Core::API::Log qw ( GetLog );
use DE_EPAGES::Database::API::Connection qw ( GetCurrentDBHandle );
use DE_EPAGES::DataCache::API::Cache qw ( UpdateCache );
use DE_EPAGES::WebInterface::API::MessageCenter qw( SynchronizeCache );


our @EXPORT_OK = qw (
    DAL_DeleteUncomfirmedNewsletterAddressee
);

my $DO_CACHE = 1;

sub _CallDriver {
    my ($FunctionName, @Parameter) = @_;
    my $dbi = GetCurrentDBHandle();
    my $Package = 'DE_EPAGES::Newsletter::DAL::'.$dbi->driverName.'::Newsletter';
    UsePackage($Package);
    GetLog->debug("call $Package->$FunctionName()");
    return $Package->$FunctionName($dbi, @Parameter);
}

sub DAL_DeleteUncomfirmedNewsletterAddressee {
    my ($Days) = @_;
    my $cacheKey = GetCurrentDBHandle()->cacheKey;
    my $dbi = GetCurrentDBHandle();

    my $aaNewsletterIDs = _CallDriver('GetIDsForUncomfirmedNewsletterAddressee', $Days);
    _CallDriver('DeleteUncomfirmedNewsletterAddressee', $Days);
    $dbi->transaction(
    sub {
        if ($DO_CACHE) {
            foreach my $aRow ( @{$aaNewsletterIDs} ) {
                foreach my $NewsletterID (@$aRow) {
                    UpdateCache($cacheKey, 'InfoNewsletteraddressee', $NewsletterID);
                }
            }
            UpdateCache($cacheKey, 'AllNewsletteraddressee');
        }
    });

    SynchronizeCache();

    return;
}

1;
