use strict;

use Getopt::Long;
use DE_EPAGES::Database::API::Connection qw ( RunOnStore );
use DE_EPAGES::Newsletter::DAL::Newsletter qw( DAL_DeleteUncomfirmedNewsletterAddressee );
use DE_EPAGES::Core::API::Script qw ( RunScript );
use Time::HiRes qw ( time );
#==================================================================================
sub Main {
    my $Help;
    my $StoreName;
    my $Password;
    my $Days;

    GetOptions(
        'storename=s'  => \$StoreName,
        'help'         => \$Help,
        'passwd=s'     => \$Password,
        'days=i'     => \$Days,
    );

    usage() if $Help or not defined $StoreName or not defined $Days;

    my $time1 = time;

    RunOnStore(
        'Store'      => $StoreName,
        'DBPassword' => $Password,
        'Sub'        => sub {

            DAL_DeleteUncomfirmedNewsletterAddressee($Days);

        }
    );

    my $time2   = time;
    my $seconds = $time2 - $time1;
    printf( "need %.3f seconds\n", $seconds );
}
#===================================================================================
sub usage {
    print <<END_USAGE;
Usage:
    perl $0 [options] [flags]

options:
    -storename  storname                            (required)
    -passwd     database user password              ()
    -days       days we substract from the current date, to check if creation date is older than substracted date (required)

flags:
    -help       show the command line options


Example:
    perl deleteUnconfirmedNewsletterSubscribtions.pl -storename Store -days 7
END_USAGE
    exit 2;
}
#==================================================================================
RunScript( 'Sub' => \&Main );

1;
