#========================================================================================
# package      DE_EPAGES::Presentation::API::Attributes::ReCaptcha_Ext
# state        private
# description  Attribute interface for Recaptcha public and private key.
#========================================================================================
package DE_EPAGES::Presentation::API::Attributes::Recaptcha_Ext;
use base qw( DE_EPAGES::Object::API::Attributes::DefaultAttribute );

use strict;
use feature 'switch';

use DE_EPAGES::Presentation::API::Constants qw( RECAPTCHA_PUBLIC_KEY RECAPTCHA_PRIVATE_KEY);

sub getAttributes {
    my $self = shift;
    my ($Object, $aNames) = @_;

    my $hValues = {};
    foreach my $Name ( @$aNames ) {
        given ($Name) {

            my $hSuperValues = $self->SUPER::getAttributes($Object, ['_ReCaptchaPublicKey', '_ReCaptchaPrivateKey']);

            when ('ReCaptchaPrivateKey') {
                if (defined $hSuperValues->{'_ReCaptchaPrivateKey'}) {
                    $hValues->{$Name} = $hSuperValues->{'_ReCaptchaPrivateKey'};
                }
                else {
                    $hValues->{$Name} = RECAPTCHA_PRIVATE_KEY();
                }
            }
            when ('ReCaptchaPublicKey') {
                if (defined $hSuperValues->{'_ReCaptchaPublicKey'}) {
                    $hValues->{$Name} = $hSuperValues->{'_ReCaptchaPublicKey'};
                }
                else {
                    $hValues->{$Name} = RECAPTCHA_PUBLIC_KEY();
                }
            }
        }
    }
    return $hValues;
}

sub setAttributes {
    my $self = shift;
    my ($Object, $hUpdate) = @_;

    if ( exists $hUpdate->{'ReCaptchaPrivateKey'} ) {
        $Object->set({'_ReCaptchaPrivateKey' => $hUpdate->{'ReCaptchaPrivateKey'}});
    }
    if ( exists $hUpdate->{'ReCaptchaPublicKey'} ) {
        $Object->set({'_ReCaptchaPublicKey' => $hUpdate->{'ReCaptchaPublicKey'}});
    }

    return;
}

1;
