#========================================================================================
# §package      DE_EPAGES::Presentation::DAL::ObjectPosition;
# §state        private
# §description  database interface for quick update of position values in object table
#========================================================================================
package DE_EPAGES::Presentation::DAL::ObjectPosition;

use base qw (Exporter);

use strict;
our @EXPORT = qw (
    DAL_RenumberObjectsBySiteID
);

use DE_EPAGES::Core::API::Log qw ( GetLog );
use DE_EPAGES::Database::API::Connection qw (  GetCurrentDBHandle );

my $DO_CACHE = 1;

sub _CallDriver {
    my ($FunctionName, @Parameter) = @_;
    my $dbi = GetCurrentDBHandle();
    my $Package = 'DE_EPAGES::Presentation::DAL::'.$dbi->driverName.'::ObjectPosition';
    DE_EPAGES::Core::API::PerlTools::UsePackage($Package);
    GetLog->debug("call $Package->$FunctionName()");
    return $Package->$FunctionName($dbi, @Parameter);
}

#========================================================================================
# §function     DAL_RenumberObjectsBySiteID
# §state        public
#----------------------------------------------------------------------------------------
# §syntax       DAL_RenumberObjectsBySiteID($SiteID, $ParentID, $StepSize);
#----------------------------------------------------------------------------------------
# §description  renumbers position of child objects for a specific object in a site.
#               renumber is done directly on database for high performance
#----------------------------------------------------------------------------------------
# §input        $SiteID | id of site | integer
# §input        $ParentID | id of parent object | integer
# §input        $StepSize | step size e.g. 10 | integer
#========================================================================================
sub DAL_RenumberObjectsBySiteID ($$$) {
    my ($SiteID, $ParentID, $StepSize) = @_;

    return _CallDriver('RenumberObjectsBySiteID', $SiteID, $ParentID, $StepSize);
}

1;

