#========================================================================================
# §package      DE_EPAGES::Presentation::DAL::mysql::ObjectPosition;
# §state        private
# §description  database interface for quick update of position values in object table
#========================================================================================
package DE_EPAGES::Presentation::DAL::mysql::ObjectPosition;

use strict;

use DE_EPAGES::DataCache::API::Cache qw (UpdateCache);

#========================================================================================
# §function     RenumberObjectsBySiteID
# §state        public
#----------------------------------------------------------------------------------------
# §syntax       DE_EPAGES::Presentation::DAL::mysql::ObjectPosition->RenumberObjectsBySiteID($dbi, $SiteID, $ParentID, $StepSize);
#----------------------------------------------------------------------------------------
# §description  renumbers all products position in a site
#               renumber is done on database for high performance
#----------------------------------------------------------------------------------------
# §input        $SiteID | id of site | integer
# §input        $ParentID | id of parent object | integer
# §input        $StepSize | step size e.g. 10 | integer
#========================================================================================
sub RenumberObjectsBySiteID {
    my $self = shift;
    my ($dbi, $SiteID, $ParentID, $StepSize) = @_;

    my $sql_set_variable = 'SET @row_number=0';
    my $sql_renumber = 'UPDATE object ' .
        'SET position=(@row_number:=@row_number+?) ' .
        'WHERE siteid=? AND parentid=?' .
        'ORDER BY position ASC';
    my $affectedObjectIDsQuery = 'SELECT objectid ' .
        'FROM object ' .
        'WHERE siteid=? AND parentid=?';

    $dbi->do($sql_set_variable);
    $dbi->do($sql_renumber, [$StepSize, $SiteID, $ParentID], ['Integer', 'Integer', 'Integer']);
    my $aAffectedObjectIDs = $dbi->execute($affectedObjectIDsQuery, [$SiteID, $ParentID], ['Integer', 'Integer']);

    # update db cache
    my $DAL_PACKAGE = DE_EPAGES::Database::API::DAL->load('DE_EPAGES::Object::API::Object::Object');

    foreach my $objectID (@$aAffectedObjectIDs) {
        UpdateCache($dbi->cacheKey, 'Object', $objectID->[0]);
        UpdateCache($dbi->cacheKey, 'InfoObject', $objectID->[0]);
        $DAL_PACKAGE->cacheUpdate($objectID->[0], undef, 0);
    }

    return 1;
}

1;
