#========================================================================================
# package      DE_EPAGES::Presentation::t::CheckCaptcha
# state        public
#----------------------------------------------------------------------------------------
# description  Provides tests for Google Captcha.
#========================================================================================
package DE_EPAGES::Presentation::t::CheckCaptcha;
use base DE_EPAGES::Core::API::Test;

use strict;
use DE_EPAGES::Presentation::UI::Object;

use Test::More;
use Test::Exception;;
use Test::Mockify;
use Test::Mockify::Verify;
use DE_EPAGES::Test::Mock::MockifyEpagesObject;
use DE_EPAGES::Test::Mock::MockifyServlet;

use JSON;

sub test {
    Test_CheckWrongCaptcha();
    Test_CheckTrueCaptcha();
    Test_CheckFalseDomainNameCaptcha();
    Test_CheckFalseSuccessCodeCaptcha();
    done_testing();
}

sub Test_CheckWrongCaptcha {
    my $SubTestName = (caller(0))[3];
    my $Servlet = _createServlet();
    my $GoogleResponse = {};

    dies_ok(
        sub {
            DE_EPAGES::Presentation::UI::Object->CheckGoogleCaptcha($Servlet, $GoogleResponse);
        }, "$SubTestName - False GoogleCaptcha"
    );
}

sub Test_CheckTrueCaptcha {
    my $SubTestName = (caller(0))[3];
    my $Servlet = _createServlet();
    my $GoogleResponse = {
        'hostname' => 'test.vm-intern.epages.com',
        'success' => JSON::true,
    };

    lives_ok(
        sub {
            DE_EPAGES::Presentation::UI::Object->CheckGoogleCaptcha($Servlet, $GoogleResponse)
        }, "$SubTestName - True GoogleCaptcha"
    )
}

sub Test_CheckFalseDomainNameCaptcha {
    my $SubTestName = (caller(0))[3];
    my $Servlet = _createServlet();
    my $GoogleResponse = {
        'hostname' => 'false.vm-intern.epages.com',
        'success' => JSON::true,
    };

    dies_ok(
        sub {
            DE_EPAGES::Presentation::UI::Object->CheckGoogleCaptcha($Servlet, $GoogleResponse)
        }, "$SubTestName - False DomainName GoogleCaptcha"
    );
}

sub Test_CheckFalseSuccessCodeCaptcha {
    my $SubTestName = (caller(0))[3];
    my $Servlet = _createServlet();
    my $GoogleResponse = {
        'hostname' => 'test.vm-intern.epages.com',
        'success' => JSON::false,
    };

    dies_ok(
        sub {
            DE_EPAGES::Presentation::UI::Object->CheckGoogleCaptcha($Servlet, $GoogleResponse)
        }, "$SubTestName - False success code GoogleCaptcha"
    )
}

#----------------------------------------------------------------------------------------
sub _createForm {

    my $MockedForm = DE_EPAGES::Test::Mock::Mockify->new('DE_EPAGES::Presentation::API::Form');

    $MockedForm->mock('executeFormError', sub {
        my $self = shift;
        my ($hErrorInfo) = @_;

        my $ErrorReason = $hErrorInfo->{'Reason'};

        if ($ErrorReason eq "RECAPTCHA_WRONG") {
            die;
        }
    });

    return $MockedForm->getMockObject();
}

#----------------------------------------------------------------------------------------
sub _createServlet {
    my ($Basket) = @_;

    my $Shop = _createShop();

    my $MockifyServlet = DE_EPAGES::Test::Mock::MockifyServlet->new($Shop);

    $MockifyServlet->addMockWithReturnValue('form', _createForm());

    return $MockifyServlet->getMockObject();
}

#----------------------------------------------------------------------------------------
sub _createShop {

    my $hAttributes = {
        'Alias' =>'DemoShop',
        'DomainName' => 'test.vm-intern.epages.com',
        'ProtocolAndServer' => 'test.vm-intern.epages.com',
    };

    my $MockedShop = DE_EPAGES::Test::Mock::MockifyEpagesObject->new('DE_EPAGES::Shop::API::Object::Shop', $hAttributes);

    my $Shop  = $MockedShop->getMockObject();
    $Shop->set({'Site' => $Shop});

    return $Shop;
}

#----------------------------------------------------------------------------------------
__PACKAGE__->runScript();

1;
