#==============================================================================
# package      DE_EPAGES::ProductRating::UI::Product
# state        private
#==============================================================================
package DE_EPAGES::ProductRating::UI::Product;
use base DE_EPAGES::Presentation::UI::Object;

use strict;
use DE_EPAGES::Core::API::Error qw ( GetError ExistsError );
use DE_EPAGES::Core::API::Log qw ( LogError );
use DE_EPAGES::Object::API::Factory qw ( LoadClassByAlias );
use DE_EPAGES::Object::API::Table::Locale qw( GetInfoLocale );
use DE_EPAGES::Object::API::Table::Language qw (GetInfoLanguage);
use DE_EPAGES::ShopMailType::API::ShopMailType qw( GetShopMailType );


#==============================================================================
# Event handlers
#==============================================================================

sub InsertProductRating {
    my $self = shift;
    my ($Servlet) = @_;
    my $Product = $Servlet->object;
    my $Form = $Servlet->form->form( $Product, 'ProductRating' );

    $self->GoogleCaptcha($Servlet);

    #spam check - check if field senderName has value,
    #this field is only for spam protection it is hidden for the customer
    return if $Form->{'SenderName'};

    # use SuperProduct if variation
    $Product = $Product->get('SuperProduct') if defined $Product->get('SuperProduct');

    # test if we have a productrating with the same parameters for this product
    my $ahRatings = $Product->get('Ratings');
    my @Ratings = grep {($_->get('Name')    eq $Form->{'Name'})   and
                        ($_->get('EMail')   eq $Form->{'EMail'})  and
                        ($_->get('Title')   eq $Form->{'Title'})  and
                        ($_->get('Comment') eq $Form->{'Comment'})
                       } @$ahRatings;

    $Servlet->vars('RatingExists', 1) if scalar @Ratings;
    return if scalar @Ratings;

    my $Shop = $Product->getSite();
    my $IsVisible = $Shop->get('ProductRatingImmediatelyVisible') ? 1:0;
    my $RatingFolder = $Shop->folder('Ratings', {'Position'=>180});
    my $User = $Servlet->user;

    my $ProductRating = LoadClassByAlias('ProductRating')->insertObject({
        'Parent' => $RatingFolder,
        'User'   => $Servlet->user,
        'Alias' =>  $RatingFolder->nextAlias,
        'Product' => $Product,
        'Rating' => $Form->{'Rating'},
        'IsVisible' => $IsVisible,
        'Name'  => $Form->{'Name'},
        'Title' => $Form->{'Title'},
        'Comment'=>$Form->{'Comment'},
        'EMail'=>  $Form->{'EMail'},
        'ShowName'=>  $Form->{'ShowName'}
    });

    # create a session to make sure the user does not see cached ratings anymore
    $Servlet->createAnonymousUser($Shop) unless defined $User;

        # set the cache to invalid
    $Shop->invalidatePageCache;

    # send the admin mail
    _SendMail($Servlet,$ProductRating);

    $Servlet->vars('RatingSuccess', 1);
}

sub _SendMail {
    my ($Servlet, $Rating) = @_;

    my $Shop = $Rating->getSite();
    my $MailType  = GetShopMailType( $Shop, 'NewProductRating', 1 );
    return unless $MailType;

    #Create Mail
    my $hMailParams = { 'HTMLAllowed' => 1 };
    my %Vars = ( 'Rating' => $Rating->tleHash );

    my $Locale = $Shop->get('LocaleID');
    my $Language = GetInfoLanguage( GetInfoLocale($Locale)->{'LanguageID'} )->{'Code2'};

    my %RegionalParams = (
        Locale      => $Locale,
        Language    => $Language,
    );

    eval {
        $MailType->send( $Rating, $hMailParams, \%RegionalParams, \%Vars );
    };
    if (ExistsError()) {
        my $Error = GetError();
        LogError($Error);
    }

    return;
}

1;
