#========================================================================================
# package      DE_EPAGES::ShopNewsletter::DAL::Newsletter
# state        private
# description  database interface for tables:
#               <ul>
#                   <li>newsletteraddressee
#               </ul>
#========================================================================================
package DE_EPAGES::ShopNewsletter::DAL::Newsletter;
use base qw ( Exporter );

use utf8;
use strict;
our @EXPORT_OK = qw (
    DAL_GetIsSubscribed
    DAL_GetCampaignSubscribedCount
    DAL_GetCampaignUnsubscribedCount
);

use DE_EPAGES::Database::API::Connection qw ( GetCurrentDBHandle );
use DE_EPAGES::ShopNewsletter::DAL::SQL::Newsletter qw ( GetCampaignSubscribedCount GetCampaignUnsubscribedCount );

#========================================================================================
# function     DAL_GetIsSubscribed
#----------------------------------------------------------------------------------------
# syntax       $isEmailSubscribed = DAL_GetIsSubscribed($newsletterCampaignID, $email);
#----------------------------------------------------------------------------------------
# example      $isEmailSubscribed = DAL_GetIsSubscribed(42217, 'oliverzscheyge@gmail.com');
#----------------------------------------------------------------------------------------
# description  Returns whether a email address is subscribed (and confirmed)
#               for a newsletter campaign.
#----------------------------------------------------------------------------------------
# input        $newsletterCampaignID | Newsletter campaign ID | int
# input        $email | Email address to check | string
# return       $isSubscribed | 1 if email is subscribed, 0 if not | int/bool
#========================================================================================
sub DAL_GetIsSubscribed {
    my ($newsletterCampaignID, $email) = @_;
     my $dbi = GetCurrentDBHandle();
    return DE_EPAGES::ShopNewsletter::DAL::SQL::Newsletter->GetIsSubscribed($dbi, $newsletterCampaignID, $email);
}

#========================================================================================
# function     DAL_GetCampaignSubscribedCount
#----------------------------------------------------------------------------------------
# syntax       $subscribedCount = DAL_GetCampaignSubscribedCount($newsletterCampaignID);
#----------------------------------------------------------------------------------------
# example      $subscribedCount = DAL_GetCampaignSubscribedCount(42217);
#----------------------------------------------------------------------------------------
# description  Returns the amount of subscribers to given newsletter campaign.
#----------------------------------------------------------------------------------------
# input        $newsletterCampaignID | Newsletter campaign ID | int
# return       $subscribed | The amount of subscribers to the campaign | int
#========================================================================================
sub DAL_GetCampaignSubscribedCount {
    my $NewsletterCampaignID = shift;
    my $Result = GetCampaignSubscribedCount($NewsletterCampaignID);
    return $Result;
}

#========================================================================================
# function     DAL_GetCampaignUnsubscribedCount
#----------------------------------------------------------------------------------------
# syntax       $unsubCount = DAL_GetCampaignUnsubscribedCount($newsletterCampaignID);
#----------------------------------------------------------------------------------------
# example      $unsubCount = DAL_GetCampaignUnsubscribedCount(42217);
#----------------------------------------------------------------------------------------
# description  Returns the amount of unsubscribers to given newsletter campaign.
#----------------------------------------------------------------------------------------
# input        $newsletterCampaignID | Newsletter campaign ID | int
# return       $unsubscribed | The amount of unsubscribers to the campaign | int
#========================================================================================
sub DAL_GetCampaignUnsubscribedCount {
    my $NewsletterCampaignID = shift;
    my $Result = GetCampaignUnsubscribedCount($NewsletterCampaignID);
    return $Result;
}
1;
