#==============================================================================
# package      DE_EPAGES::ShopNewsletter::UI::Shop
# state        private
#==============================================================================
package DE_EPAGES::ShopNewsletter::UI::Shop;
use base DE_EPAGES::Newsletter::UI::Site;

use strict;
use DE_EPAGES::Core::API::PerlTools qw(Compare);
use DE_EPAGES::ShopMailType::API::ShopMailType qw( GetShopMailType );
use DE_EPAGES::Newsletter::API::Constants qw(NEWSLETTER_SENT);
use DE_EPAGES::ShopNewsletter::API::Newsletter qw ( GetSubscribeCount GetUnsubscribeCount );

# override
sub getMailType {
    my $self = shift;
    my( $Shop, $Alias ) = @_;

    return GetShopMailType( $Shop, $Alias, 1 )
}

sub SaveNewsletterInfoText {
    my $self    = shift;
    my ($Servlet) = @_;
    my $Shop    = $Servlet->object;

    # localized InfoText
    my $hFormValuesLanguage = $Servlet->form->form($Shop, 'NewsletterInformation');
    foreach my $hLanguage (@{$hFormValuesLanguage->{'Languages'}}) {
        $Shop->set({'NewsletterInfoText' => $hLanguage->{'NewsletterInfoText'}}, $hLanguage->{'LanguageID'});
    }
    foreach my $hLanguage (@{$hFormValuesLanguage->{'CheckoutLanguages'}}) {
        $Shop->set({'NewsletterTextSubscrInCheckout' => $hLanguage->{'NewsletterTextSubscrInCheckout'}}, $hLanguage->{'CheckoutLanguageID'});
    }
    $Shop->set({'NewsletterHideSubscrInCheckout' => $hFormValuesLanguage->{'NewsletterHideSubscrInCheckout'}});
}

sub DeleteNewsletterCampaigns {
    my $self = shift;
    my ($Servlet) = @_;
    my $Shop = $Servlet->object;

    $self->SUPER::DeleteListedObjects($Servlet);
    $Shop->folder('NewsletterCampaigns')->renumberChildren;
}

sub ExportNewsletterAddressee {
    my $self = shift;
    my ($Servlet) = @_;

    my $Shop = $Servlet->object;
    my $hValues = $Servlet->form->form($Shop, 'ListedObjects');
    my @ListObjects = map { $Shop->loadObject( $_->{'ListObjectID'}, 'NewsletterCampaign' ) } @{$hValues->{'ListObjectIDs'}};
    return unless scalar @ListObjects;

    $Servlet->vars('SelectedObjects', [map {$_->tleHash} @ListObjects]);
    $Servlet->vars('CSVExportImportType', 'NewsletterAddressee');
    $Servlet->vars('EXPORT_SELECTED_OBJECTS', 1);
    $Servlet->vars('ViewActionAbort', $Servlet->params('ViewAction'));
    $Servlet->vars('ObjectIDAbort', $Shop->id);
    $Servlet->vars('ViewAction', 'MBO-ViewCSVExportImportNewsletterAddressee');
    $Servlet->vars('ObjectID', $Shop->getSite->id);
    return;
}

sub MBOViewNewsletterWidget {
    my $self = shift;
    my ($Servlet) = @_;
    my $Shop = $Servlet->object;

    $self->BlockMonitor($Servlet);

    # get subscribed and unsubscribed addressees from all newsletter campaigns

    # get all campaigns
    my $CumulatedSubscribedCount = 0;
    my $CumulatedUnsubscribedCount = 0;
    my $NLSentCount = 0;
    my $LastSentRecipientsCount = 0;
    my $LastSentDate;

    my $ahCampaigns = $Shop->get('NewsletterCampaigns');

    # details for campaign
    foreach my $hCampaign (@$ahCampaigns){
        my $NewsletterCampaignID = $hCampaign->get('NewsletterCampaignID');
        my $SubscribedAddressees = GetSubscribeCount($NewsletterCampaignID);
        $CumulatedSubscribedCount += $SubscribedAddressees;
        my $UnsubscribedAddressees = GetUnsubscribeCount($NewsletterCampaignID);
        $CumulatedUnsubscribedCount += $UnsubscribedAddressees;

        # details of NL
        foreach my $hNewsletter (@{$hCampaign->get('Newsletters')}){
            if ($hNewsletter->get('Status') == NEWSLETTER_SENT) {
                $NLSentCount ++;
                #get the sent date of the latest sent NL
                my $hNLSentDate = $hNewsletter->get('SentDate');
                $LastSentDate =  $hNLSentDate unless defined $LastSentDate;
                if ( -1 != Compare('DateTime', $hNewsletter->get('SentDate'), $LastSentDate)){
                    $LastSentDate = $hNLSentDate;
                    $LastSentRecipientsCount = scalar @{$hNewsletter->get('Recipients')};
                }
            }
        }
    }

    $Servlet->vars('CampaignSubscribedCount', $CumulatedSubscribedCount);
    $Servlet->vars('CampaignUnSubscribedCount', $CumulatedUnsubscribedCount);
    $Servlet->vars('NLSentCount', $NLSentCount);
    $Servlet->vars('NLLastSentDate', $LastSentDate);
    $Servlet->vars('NLLastSentRecipientsCount', $LastSentRecipientsCount);

    return;
}

1;
